"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const json2php_1 = __importDefault(require("./json2php"));
const U = 'LegoMirrorUtil';
var CodeType;
(function (CodeType) {
    CodeType[CodeType["line"] = 0] = "line";
    CodeType[CodeType["raw"] = 1] = "raw";
})(CodeType || (CodeType = {}));
function isPointer(s) {
    return typeof s === 'string' && /^#\//.test(s);
}
function getPath(str) {
    return str.slice(2).split('/').filter(a => a);
}
const tracer = 'ijkhlmn';
class CodeBuffer {
    constructor(options) {
        this.buffer = [{
                type: CodeType.line,
                code: '$newData = array()'
            }];
        this.trackerIndex = 0;
        this.root = options.root || '$tplData';
    }
    transformGetter(path, variable) {
        return `${variable}${path.map(p => `[${/^\$/.test(p) ? p : json2php_1.default(p)}]`).join('')}`;
    }
    transfromSetter(to, setter, content) {
        return `${U}::set(${setter}, array(${to.map(t => /^\$/.test(t) ? t : json2php_1.default(t)).join(', ')}), ${content})`;
    }
    getParams(parentPath) {
        let getter = this.root;
        let setter = '$newData';
        if (parentPath && parentPath.from.length > 0) {
            getter = this.transformGetter(parentPath.from, getter);
        }
        if (parentPath && parentPath.to.length > 0) {
            setter = this.transformGetter(parentPath.to, setter);
        }
        return { getter, setter };
    }
    fromPath(from, to, parentPath) {
        const { getter, setter } = this.getParams(parentPath);
        this.buffer.push({
            type: CodeType.line,
            code: this.transfromSetter(to, setter, this.transformGetter(from, getter))
        });
    }
    fromConstant(constant, to, parentPath) {
        const { setter } = this.getParams(parentPath);
        this.buffer.push({
            type: CodeType.line,
            code: this.transfromSetter(to, setter, json2php_1.default(constant))
        });
    }
    addCat(from, to, data, parentPath) {
        const dataContent = isPointer(data) ? this.transformGetter(getPath(data), this.root) : json2php_1.default(data);
        const { setter } = this.getParams(parentPath);
        if (isPointer(from)) {
            const getter = this.transformGetter(getPath(from), this.root);
            this.buffer.push({
                type: CodeType.line,
                code: this.transfromSetter(to, setter, `(isset(${getter}) ? ${getter} : '') . (${dataContent})`)
            });
        }
        else {
            this.buffer.push({
                type: CodeType.line,
                code: this.transfromSetter(to, setter, `(${json2php_1.default(data)}) . (${dataContent})`)
            });
        }
    }
    addCopy(from, to, data, parentPath) {
        const { $type = 'object', $mirror } = data;
        if (!$mirror) {
            isPointer(from)
                ? this.fromPath(getPath(from), to, parentPath)
                : this.fromConstant(from, to, parentPath);
            return;
        }
        if (!parentPath) {
            parentPath = { from: [], to: [] };
        }
        if ($type === 'object' && isPointer(from)) {
            const fromPath = getPath(from);
            const { setter, getter } = this.getParams({ from: fromPath, to });
            this.buffer.push({
                type: CodeType.raw,
                code: `if (isset(${getter})) {`
            });
            this.buffer.push({
                type: CodeType.raw,
                code: `if (!isset(${setter})) {`
            });
            this.fromConstant([], to, parentPath);
            this.buffer.push({
                type: CodeType.raw,
                code: `}`
            });
            this.walk($mirror, {
                from: [...parentPath.from, ...fromPath],
                to: [...parentPath.to, ...to]
            });
            this.buffer.push({
                type: CodeType.raw,
                code: `}`
            });
            return;
        }
        if ($type === 'object') {
            this.fromConstant(from, to, parentPath);
            return;
        }
        if ($type === 'array') {
            const fromPath = getPath(from);
            const { setter, getter } = this.getParams({
                from: [...parentPath.from, ...fromPath],
                to: [...parentPath.to, ...to
                ]
            });
            this.buffer.push({
                type: CodeType.raw,
                code: `if (isset(${getter})) {`
            });
            this.buffer.push({
                type: CodeType.raw,
                code: `if (!empty(${getter}) && !isset(${getter}[0])) {\n${getter} = array(${getter});\n}`
            });
            if (data.$maxItems) {
                this.buffer.push({
                    type: CodeType.raw,
                    code: `${getter} = array_slice(${getter}, 0, ${data.$maxItems});`
                });
            }
            const tracerVar = `$${tracer[this.trackerIndex]}`;
            this.trackerIndex++;
            this.buffer.push({
                type: CodeType.raw,
                code: `foreach (${getter} as ${tracerVar} => $item) {`
            });
            this.buffer.push({
                type: CodeType.raw,
                code: `if (!isset(${setter}[${tracerVar}])) {`
            });
            this.fromConstant([], [...to, tracerVar], parentPath);
            this.buffer.push({
                type: CodeType.raw,
                code: `}`
            });
            this.walk($mirror, {
                from: [...parentPath.from, ...fromPath, tracerVar],
                to: [...parentPath.to, ...to, tracerVar]
            });
            this.buffer.push({
                type: CodeType.raw,
                code: `}\n}`
            });
        }
    }
    walk(mirror, parentPath) {
        for (let [to, from] of Object.entries(mirror)) {
            const toPath = getPath(to);
            if (typeof from === 'string') {
                isPointer(from)
                    ? this.fromPath(getPath(from), toPath, parentPath)
                    : this.fromConstant(from, toPath, parentPath);
            }
            else if (['number', 'boolean', 'string'].includes(typeof from)
                || Array.isArray(from)
                || !from.$from) {
                this.fromConstant(from, toPath, parentPath);
            }
            else if (typeof from === 'object' && from.$from) {
                const { $from, $action = 'copy', $data } = from;
                switch ($action) {
                    case 'cat':
                        if (typeof $data !== 'object' && typeof $from === 'string') {
                            this.addCat($from, toPath, $data, parentPath);
                        }
                        break;
                    case 'copy':
                        if (typeof $data === 'object') {
                            this.addCopy($from, toPath, $data, parentPath);
                        }
                    default:
                        break;
                }
            }
        }
    }
    toString() {
        return this.buffer.map(a => a.code + (a.type === CodeType.line ? ';' : '')).join('\n');
    }
}
exports.default = CodeBuffer;
//# sourceMappingURL=CodeBuffer.js.map